# -*- coding: utf-8 -*-
from types import MappingProxyType

# Deprecated:
# To avoid affecting old interfaces, this constant is introduced to represent "delete/clear".
# It can continue to be used in old interfaces if needed.
# For new interfaces, please use DEFAULT instead
OFF = False
ON = True

waf_rule_sets_name2id = {
    'robots': 1,
    'generic': 2,
    'inject': 3,
    'trojans': 4,
    'xss': 5,
    'scanner_detection': 6,
    'protocol_enforcement': 7,
    'protocol_attack': 8,
    'application_attack_lfi': 9,
    'application_attack_rfi': 10,
    'application_attack_rce': 11,
    'application_attack_php': 12,
    'application_attack_nodejs': 13,
    'application_attack_xss': 14,
    'application_attack_sqli': 15,
    'application_attack_session_fixation': 16,
    'application_attack_java': 17,
}

valid_waf_thresholds = {
    'high': True,
    'medium': True,
    'low': True,
    'none': True,
}

cross_req_waf_threshold_scores = {
    'high': 1000,
    'medium': 100,
    'low': 10,
}

waf_threshold_scores = {
    'high': 12,
    'medium': 6,
    'low': 3,
}

ERROR_LOG_LEVELS = MappingProxyType({
    'stderr': 0,
    'emerg': 1,
    'alert': 2,
    'crit': 3,
    'error': 4,
    'warn': 5,
    'notice': 6,
    'info': 7,
    'debug': 8,
})


class Sentinel:
    """
    Sentinel is a singleton class that is used to indicate that a parameter is not provided.
    Sentinel: parameter is not provided
    None: parameter is provided and its value is None
    Any other value: parameter is provided and its value is not None
    """
    pass

DEFAULT = Sentinel()

INITIAL_EDGE_VERSION = 110
EDGE_VERSION_24_09_16 = 24091601
EDGE_VERSION_24_09_06 = 24090601



# -*- coding: utf-8 -*-
# import io
import os
import random
import sdk_test
from edge2client.constants import OFF, ON

CURPATH = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))


class TestRuleOrder(sdk_test.TestSdk):
    def test_rule_order(self):
        condition = [{'var': 'uri', 'op': 'eq', 'val': '/favicon.ico'}]
        conseq = [
            {'print': {
                'msg': 'hello'
            }}
        ]

        # new rule 1
        rule_id = self.client.new_rule(condition=condition, conseq=conseq)
        self.assertIs(type(rule_id), int)
        self.assertGreater(rule_id, 0)

        rule_id1 = rule_id

        rule = self.client.get_rule(rule_id)
        self.assertEqual(rule['order'], 0)

        # new rule 2
        rule_id = self.client.new_rule(condition=condition, conseq=conseq)
        self.assertIs(type(rule_id), int)
        self.assertGreater(rule_id, 0)

        rule_id2 = rule_id

        rule = self.client.get_rule(rule_id)
        self.assertEqual(rule['order'], 0)

        # new rule 3
        rule_id = self.client.new_rule(condition=condition, conseq=conseq, order=1)
        self.assertIs(type(rule_id), int)
        self.assertGreater(rule_id, 0)

        rule_id3 = rule_id

        rule = self.client.get_rule(rule_id)
        self.assertEqual(rule['order'], 1)

        conseq = {
            'user-code': {'el': 'true => exit(403);'}
        }

        # update rule 1, order does not change
        ok = self.client.put_rule(rule_id=rule_id1, conseq=conseq)
        self.assertEqual(ok, True)

        rule = self.client.get_rule(rule_id1)
        self.assertEqual(rule['order'], 0)

        # update rule 3, order does not change
        ok = self.client.put_rule(rule_id=rule_id, conseq=conseq)
        self.assertEqual(ok, True)

        rule = self.client.get_rule(rule_id)
        self.assertEqual(rule['order'], 1)

        # update rule 1, order changed
        ok = self.client.put_rule(rule_id=rule_id1, conseq=conseq, order=2)
        self.assertEqual(ok, True)

        rule = self.client.get_rule(rule_id1)
        self.assertEqual(rule['order'], 2)



import os

from .utils import error, warn, info, md5sum, line
from .read_config import read_lua_modules

def cleanup_global_lua_modules(ctx):
    client = ctx['client']
    partition_id = ctx['partition_id']
    lua_module_sync_to_all = ctx['lua_module_sync_to_all']

    if lua_module_sync_to_all:
        return

    modules = client.get_all_partition_lua_module(partition_id)
    for mod in modules:
        if 'id' in mod:
            try:
                info(f"removing global lua module, partition id: {partition_id}, module id: {mod['id']}")
                client.del_partition_lua_module(partition_id, mod['id'])
            except Exception as e:
                error(f"Failed to partition lua module \"{name}\", lua module id: {mod['id']}", e)

def process_global_lua_modules(ctx):
    client = ctx['client']
    partition_id = ctx['partition_id']
    configs_path = ctx['configs_path']
    lua_module_sync_to_all = ctx['lua_module_sync_to_all']
    location = ctx.get('location', None)

    if not location:
        location = "global_lua_modules"

    # read local lua modules
    new_lua_modules = read_lua_modules(configs_path, location)
    if new_lua_modules is None:
        return

    if not new_lua_modules:
        warn(f"no lua modules were found in directory \"{location}\".")
        if lua_module_sync_to_all:
            # do not delete if sync to all
            return

    info("Checking if lua modules have changed")

    # get old lua module
    old_lua_modules = client.get_all_partition_lua_module(partition_id)
    old_lua_module_names = {}
    # check if lua module changed
    if old_lua_modules:
        for mod in old_lua_modules:
            # print(mod)
            old_lua_module_names[mod['name']] = mod

    # update or insert lua module
    sorted_filenames = sorted(new_lua_modules.keys())
    for mod_name in sorted_filenames:
        mod_code = new_lua_modules[mod_name]
        if mod_name in old_lua_module_names:
            # check md5
            old_mod = old_lua_module_names[mod_name]
            del old_lua_module_names[mod_name]
            if md5sum(mod_code) == md5sum(old_mod['code']):
                # check next module
                # info(f"global lua module have not changed, file: {mod_name}.lua")
                continue
            else:
                # update
                try:
                    info(f"updating global lua module, file: {mod_name}.lua")
                    client.put_partition_lua_module(partition_id, old_mod['id'], mod_name, mod_code)
                except Exception as e:
                    error(f"failed to update global lua module, file: {mod_name}.lua", e)
        else:
            # insert
            try:
                info(f"adding global lua module: {mod_name}.lua")
                client.new_partition_lua_module(partition_id, mod_name, mod_code)
            except Exception as e:
                error(f"failed to add lua module, file: {mod_name}.lua", e)

    if lua_module_sync_to_all:
        # do not delete if sync to all
        return

    for name, mod in old_lua_module_names.items():
        if name not in new_lua_modules:
            try:
                info(f"Removing partition lua module \"{name}\"")
                client.del_partition_lua_module(partition_id, mod['id'])
            except Exception as e:
                error(f"Failed to partition lua module \"{name}\", lua module id: {mod['id']}", e)

def export_global_lua_modules(ctx):
    client = ctx['client']
    partition_id = ctx['partition_id']
    configs_path = ctx['export_to_path']

    modules = client.get_all_partition_lua_module(partition_id)
    if not modules:
        info(f"No global Lua modules found for partition_id: {partition_id}")
        return

    lua_modules_path = os.path.join(configs_path, "global_lua_modules")

    # Create lua_modules directory if it doesn't exist
    os.makedirs(lua_modules_path, exist_ok=True)

    for module in modules:
        module_name = module['name']
        module_code = module['code']

        file_path = os.path.join(lua_modules_path, f"{module_name}.lua")

        try:
            with open(file_path, 'w') as f:
                f.write(module_code)
            info(f"Global Lua module '{module_name}' exported successfully to global_lua_modules/{module_name}.lua")
        except Exception as e:
            error(f"Failed to export global Lua module '{module_name}' to global_lua_modules/{module_name}.lua", e)

    info(f"All global Lua modules exported to global_lua_modules/")



import os
import hashlib
import tempfile
import tarfile
import shutil

from .utils import error, warn, info, line
from .read_config import read_yaml_config, write_yaml_config

def check_global_static_files(static_files, filename, configs_path, location):
    if not isinstance(static_files, list):
        error(f"Unsupported global static files format, file: {filename}")

    valid_types = ['dir', 'file', 'archived']
    for item in static_files:
        if not isinstance(item, dict):
            error(f"Each static file item must be a dictionary, file: {filename}, line: {line(item)}")

        if 'type' in item and (item['type'] == "favicon" or item['type'] == "html"):
            item['type'] = "file"

        if 'type' not in item or item['type'] not in valid_types:
            error(f"Invalid or missing 'type' in static file item, file: {filename}, line: {line(item)}")

        if 'path' not in item or not isinstance(item['path'], str):
            error(f"Missing or invalid 'path' in static file item, file: {filename}, line: {line(item)}")

        if item['type'] != 'archived' and ('name' not in item or not isinstance(item['name'], str)):
            error(f"Missing or invalid 'name' in static file item, file: {filename}, line: {line(item)}")

        # check archived file type, only support tar series files
        if item['type'] == 'archived':
            file_path = os.path.join(configs_path, location, "files", item['path'])
            try:
                with open(file_path, 'rb') as f:
                    if not tarfile.is_tarfile(f.name):
                        error(f"Unsupported archive type, only tar files are allowed: {file_path}, line: {line(item)}")
            except Exception as e:
                error(f"Failed to check archive file type: {file_path}, line: {line(item)}, error: {str(e)}")

    return True

def get_file_md5(file_path):
    hash_md5 = hashlib.md5()
    with open(file_path, "rb") as f:
        for chunk in iter(lambda: f.read(4096), b""):
            hash_md5.update(chunk)
    return hash_md5.hexdigest()

def process_global_static_files(ctx):
    client = ctx['client']
    configs_path = ctx['configs_path']
    location = ctx.get('location', None)

    if not location:
        location = "global_static_files"

    configs = read_yaml_config(configs_path, location)
    if configs is None:
        return

    info("Checking if global static files are valid")
    for filename, static_files in configs.items():
        check_global_static_files(static_files, filename, configs_path, location)

    info("Processing global static files")

    # Get all existing files from server
    server_files = {}
    paths_to_check = [""]
    while paths_to_check:
        current_path = paths_to_check.pop(0)
        files = client.get_all_static_files(path=current_path)
        for file in files:
            server_files[file['path']] = file
            if file['type'] == 'dir':
                paths_to_check.append(file['path'])

    for filename, static_files in configs.items():
        for item in static_files:
            path = os.path.dirname(item['path'])

            if item['type'] == 'dir':
                if item['path'] not in server_files:
                    try:
                        info(f"Creating directory: {item['path']}")
                        client.create_static_file_directory(item['name'], path=path, label=item.get('label', None))
                    except Exception as e:
                        error(f"Failed to create directory: {item['path']}", e)
            elif item['type'] == 'file':
                file_path = os.path.join(configs_path, location, "files", item['path'])
                if not os.path.exists(file_path):
                    error(f"File not found: {file_path}")
                    continue

                local_md5 = get_file_md5(file_path)

                if item['path'] in server_files:
                    server_file = server_files[item['path']]
                    server_content = client.get_static_file_content(server_file['id'])
                    server_md5 = hashlib.md5(server_content).hexdigest()

                    if local_md5 != server_md5:
                        try:
                            info(f"Updating file: {item['path']}")
                            with open(file_path, 'rb') as file:
                                content = file.read()
                            client.set_static_file(server_file['id'], item['name'], content, path=path, label=item.get('label', None))
                        except Exception as e:
                            error(f"Failed to update file: {item['path']}", e)
                else:
                    try:
                        info(f"Uploading new file: {item['path']}")
                        with open(file_path, 'rb') as file:
                            content = file.read()
                        client.upload_static_file(item['name'], content, path=path, label=item.get('label', None))
                    except Exception as e:
                        error(f"Failed to upload file: {item['path']}", e)
            elif item['type'] == 'archived':
                file_path = os.path.join(configs_path, location, "files", item['path'])
                if not os.path.exists(file_path):
                    error(f"File not found: {file_path}")
                    continue

                with tempfile.TemporaryDirectory() as temp_dir:
                    try:
                        info(f"Extracting archive: {file_path}")
                        with tarfile.open(file_path, 'r:*') as tar:
                            tar.extractall(temp_dir)

                        for root, dirs, files in os.walk(temp_dir):
                            sorted_dirs = sorted(dirs)
                            for dir_name in sorted_dirs:
                                rel_path = os.path.relpath(os.path.join(root, dir_name), temp_dir)
                                full_path = os.path.join(path, rel_path)
                                if full_path not in server_files:
                                    try:
                                        info(f"Creating directory from archive: {full_path}")
                                        client.create_static_file_directory(dir_name, path=os.path.dirname(full_path))
                                    except Exception as e:
                                        error(f"Failed to create directory from archive: {full_path}", e)

                            sorted_files = sorted(files)
                            for file_name in sorted_files:
                                rel_path = os.path.relpath(os.path.join(root, file_name), temp_dir)
                                full_path = os.path.join(path, rel_path)
                                file_content_path = os.path.join(root, file_name)

                                with open(file_content_path, 'rb') as f:
                                    content = f.read()
                                    local_md5 = hashlib.md5(content).hexdigest()

                                if full_path in server_files:
                                    server_file = server_files[full_path]
                                    server_content = client.get_static_file_content(server_file['id'])
                                    server_md5 = hashlib.md5(server_content).hexdigest()

                                    if local_md5 != server_md5:
                                        try:
                                            info(f"Updating file from archive: {full_path}")
                                            client.set_static_file(server_file['id'], file_name, content,
                                                                 path=os.path.dirname(full_path))
                                        except Exception as e:
                                            error(f"Failed to update file from archive: {full_path}", e)
                                else:
                                    try:
                                        info(f"Uploading new file from archive: {full_path}")
                                        client.upload_static_file(file_name, content,
                                                               path=os.path.dirname(full_path))
                                    except Exception as e:
                                        error(f"Failed to upload file from archive: {full_path}", e)

                    except Exception as e:
                        error(f"Failed to process archive: {file_path}", e)

    # since this is a global configuration,
    # we will not perform deletion operations in order to maintain compatibility with multiple local configurations.
    # # Check for files to delete
    # local_paths = set(item['path'] for filename, static_files in configs.items() for item in static_files)
    # for server_path in server_files:
    #     if server_path not in local_paths:
    #         try:
    #             info(f"Deleting file: {server_path}")
    #             client.del_static_file(server_files[server_path]['id'])
    #         except Exception as e:
    #             error(f"Failed to delete file: {server_path}", e)

def cleanup_global_static_files(ctx):
    pass
    # since this is a global configuration,
    # we will not perform deletion operations in order to maintain compatibility with multiple local configurations.
    # client = ctx['client']

    # try:
    #     info("Removing all global static files")
    #     files = client.get_all_static_files()
    #     for file in files:
    #         file_id = file['id']
    #         client.del_static_file(file_id)
    # except Exception as e:
    #     error("Failed to remove all global static files", e)

def export_global_static_files(ctx):
    client = ctx['client']
    configs_path = ctx['export_to_path']
    location = ctx.get('location', None)

    if not location:
        location = "global_static_files"

    next_paths = [ "" ]

    formatted_static_dirs = []
    formatted_static_files = []

    while next_paths:
        path = next_paths.pop(0)
        static_files = client.get_all_static_files(path=path)
        if not static_files:
            continue

        for item in static_files:
            formatted_item = {
                'type': item['type'],
                'path': item['path'],
                'name': item['name']
            }

            work_dir=os.path.join(configs_path, location, "files")

            try:
                if item['type'] == 'dir':
                    formatted_static_dirs.append(formatted_item)
                    file_path = os.path.join(work_dir, item['path'])
                    os.makedirs(file_path, exist_ok=True)
                    next_paths.append(item['path'])
                else:
                    formatted_static_files.append(formatted_item)
                    content = client.get_static_file_content(item['id'])
                    file_path = os.path.join(work_dir, item['path'])
                    os.makedirs(os.path.dirname(file_path), exist_ok=True)
                    with open(file_path, 'wb') as file:
                        file.write(content)

                    if item['type'] != "file":
                        item['type'] = "file"

                info(f"Exported file content to: {file_path}")
            except Exception as e:
                error(f"Failed to export file content: {item['path']}", e)

    export_path = os.path.join(configs_path, location)

    formatted_static_files = formatted_static_dirs + formatted_static_files

    if formatted_static_files:
        try:
            write_yaml_config(export_path, "global_static_files.yaml", formatted_static_files)
            info("Global static files configuration exported successfully to global_static_files/global_static_files.yaml")
        except Exception as e:
            error("Failed to export global static files configuration", e)
    else:
        info("No global static files found")



# -*- coding: utf-8 -*-
# import io
import sdk_test
import time
import random

class TestAcmeProviders(sdk_test.TestSdk):
    def test_acme_providers(self):
        # create
        now = random.randint(1, int(time.time()))
        endpoint = "https://acme.zerossl.com/v2/DV90"
        email = "oredge-sdk@openresty.com"
        eab_hmac_key = "abcdefghijklmnopqrstuvwxyz"
        eab_kid = "_DPxNuSfLTqcD3331Q49wT30OaelCTWsEOJuMPf_dL4"
        provider_id = self.client.new_acme_provider(name=f"amce_providers_{now}",
                                                 endpoint=endpoint,
                                                 eab_hmac_key=eab_hmac_key,
                                                 eab_kid=eab_kid, email=email)
        self.assertIsInstance(provider_id, int)

        # get
        acme_provider = self.client.get_acme_provider(provider_id)
        self.assertEqual(acme_provider.get('name'), f"amce_providers_{now}")
        self.assertEqual(acme_provider.get('endpoint'), endpoint)
        self.assertEqual(acme_provider.get('email'), email)
        self.assertEqual(acme_provider.get('eab_hmac_key'), eab_hmac_key)
        self.assertEqual(acme_provider.get('eab_kid'), eab_kid)

        # modify
        new_time = random.randint(1, int(time.time()))
        new_endpoint = "https://acme.zerossl.com/v2/DV90new"
        new_email = "oredge-sdk-new@openresty.com"
        new_eab_hmac_key = "abcdefghijklmnopqrstuvwxyz-new"
        new_eab_kid = "_DPxNuSfLTqcD3331Q49wT30OaelCTWsEOJuMPf_dL4_new"
        ok = self.client.put_acme_provider(id=provider_id,
                                           name=f"amce_providers_{new_time}",
                                           endpoint=new_endpoint,
                                           eab_hmac_key=new_eab_hmac_key,
                                           eab_kid=new_eab_kid, email=new_email)
        self.assertTrue(ok)

        acme_provider = self.client.get_acme_provider(provider_id)
        self.assertEqual(acme_provider.get('name'), f"amce_providers_{new_time}")
        self.assertEqual(acme_provider.get('endpoint'), new_endpoint)
        self.assertEqual(acme_provider.get('email'), new_email)
        self.assertEqual(acme_provider.get('eab_hmac_key'), new_eab_hmac_key)
        self.assertEqual(acme_provider.get('eab_kid'), new_eab_kid)

        acme_providers = self.client.get_all_acme_providers()
        self.assertEqual(len(acme_providers), 1)

        # delete
        ok = self.client.del_acme_provider(id=provider_id)
        self.assertTrue(ok)

        acme_providers = self.client.get_all_acme_providers()
        self.assertEqual(len(acme_providers), 0)


    def test_use_acme_provider(self):
        # create
        now = random.randint(1, int(time.time()))
        endpoint = "https://acme.zerossl.com/v2/DV90"
        email = "oredge-sdk@openresty.com"
        eab_hmac_key = "abcdefghijklmnopqrstuvwxyz"
        eab_kid = "_DPxNuSfLTqcD3331Q49wT30OaelCTWsEOJuMPf_dL4"
        provider_id = self.client.new_acme_provider(name=f"amce_providers_{now}",
                                                 endpoint=endpoint,
                                                 eab_hmac_key=eab_hmac_key,
                                                 eab_kid=eab_kid, email=email)
        self.assertIsInstance(provider_id, int)

        # get
        acme_provider = self.client.get_acme_provider(provider_id)
        self.assertEqual(acme_provider.get('name'), f"amce_providers_{now}")
        self.assertEqual(acme_provider.get('endpoint'), endpoint)
        self.assertEqual(acme_provider.get('email'), email)
        self.assertEqual(acme_provider.get('eab_hmac_key'), eab_hmac_key)
        self.assertEqual(acme_provider.get('eab_kid'), eab_kid)

        old_app_id = self.app_id
        domains = ['prefix' + self.apex]
        app_id = self.client.new_app(domains=domains, label=self.apex)
        self.client.use_app(app_id)

        cert_id = self.client.set_le_cert(domains=domains, acme_provider=provider_id, acme_csr_type='rsa')
        self.assertIsInstance(cert_id, int)

        self.client.use_app(old_app_id)

        ok = self.client.del_app(app_id)
        self.assertTrue(ok)

        ok = self.client.del_acme_provider(id=provider_id)
        self.assertTrue(ok)



# -*- coding: utf-8 -*-
# import io
import os
import random
import sdk_test
from edge2client.constants import OFF, ON

CURPATH = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))


class TestRule(sdk_test.TestSdk):
    def test_waf(self):
        condition = [{'var': 'uri', 'op': 'prefix', 'val': '/foo'}]
        waf_rule = {'rule_sets': [7,8,9,10,11,12,13,14,15,16,17],
                    'action': '403 Forbidden', 'threshold': 'low'}
        rule_id = self.client.new_rule(condition=condition,
                                       waf=waf_rule, last=True)
        self.assertIs(type(rule_id), int)
        self.assertGreater(rule_id, 0)

        waf_rule = {'rule_sets': [7,8,9,10,11,12,13,14,15,16,17],
                    'action': '403 Forbidden', 'threshold': 'high'}
        ok = self.client.put_rule(rule_id=rule_id, waf=waf_rule,
                                  condition=condition)
        self.assertTrue(ok)

        data = self.client.get_rule(rule_id)
        self.assertIn(7, data['waf']['rule_sets'])
        self.assertIn(8, data['waf']['rule_sets'])
        self.assertEqual(data['waf']['threshold_score'], 1000)

        waf_rule = {'rule_sets': [7,8,9,10,11,12,13,14,15,16,17],
                    'action': 'log'}
        ok = self.client.put_rule(rule_id=rule_id, waf=waf_rule,
                                  condition=condition)
        self.assertTrue(ok)

        waf_rule = {'rule_sets': [7,8,9,10,11,12,13,14,15,16,17],
                    'action': 'edge-captcha', 'threshold': 'low',
                    'clearance': 60}
        ok = self.client.put_rule(rule_id=rule_id, waf=waf_rule,
                                  condition=condition)
        self.assertTrue(ok)

        waf_rule = {'rule_sets': [7,8,9,10,11,12,13,14,15,16,17],
                    'action': 'redirect', 'threshold': 'medium',
                    'redirect_url': 'https://openrsty.org'}
        ok = self.client.put_rule(rule_id=rule_id, waf=waf_rule,
                                  condition=condition)
        self.assertTrue(ok)

        data = self.client.get_rule(rule_id)
        # None means true
        self.assertEqual(data['waf']['cross_requests'], True)
        self.assertEqual(data['waf']['threshold_score'], 100)

        # use sensitivity instead of threshold
        # cross_requests = False
        waf_rule = {'rule_sets': [7,8,9,10,11,12,13,14,15,16,17],
                    'action': 'redirect',
                    'sensitivity': 'high',
                    'cross_requests': False,
                    'redirect_url': 'https://openrsty.org'}
        ok = self.client.put_rule(rule_id=rule_id, waf=waf_rule,
                                  condition=condition)
        self.assertTrue(ok)

        data = self.client.get_rule(rule_id)
        self.assertEqual(data['waf']['cross_requests'], False)
        # high sensitivity == low threshold
        self.assertEqual(data['waf']['threshold_score'], 3)

        # custom score
        waf_rule = {'rule_sets': [7,8,9,10],
                    'action': 'redirect',
                    'sensitivity': 'none',
                    'score': 1,
                    'cross_requests': False,
                    'redirect_url': 'https://openrsty.org'}
        ok = self.client.put_rule(rule_id=rule_id, waf=waf_rule,
                                  condition=condition)
        self.assertTrue(ok)

        data = self.client.get_rule(rule_id)
        self.assertEqual(data['waf']['cross_requests'], False)
        self.assertEqual(data['waf']['threshold_score'], 1)
        self.assertEqual(data['waf']['rule_sets_threshold'], [-1, -1, -1, -1])

        # custom score
        waf_rule = {'rule_sets': [7,8,9,10],
                    'action': 'redirect',
                    'sensitivity': 'none',
                    'score': 1,
                    'cross_requests': False,
                    'rule_sets_threshold': [1,2,3,4],
                    'redirect_url': 'https://openrsty.org'}
        ok = self.client.put_rule(rule_id=rule_id, waf=waf_rule,
                                  condition=condition)
        self.assertTrue(ok)

        data = self.client.get_rule(rule_id)
        self.assertEqual(data['waf']['cross_requests'], False)
        self.assertEqual(data['waf']['threshold_score'], 1)
        self.assertEqual(data['waf']['rule_sets_threshold'], [1,2,3,4])

        ok = self.client.del_rule(rule_id)
        self.assertTrue(ok)

        data = self.client.get_rule(rule_id)
        self.assertEqual(data, {})

    def test_cache(self):
        condition = [{'var': 'client-country', 'op': 'eq', 'val': 'CN'}]
        cache_key = ['uri', 'query-string', 'client-city']
        cache_rule = {'cache_key': cache_key}
        rule_id = self.client.new_rule(condition=condition, cache=cache_rule)
        self.assertIs(type(rule_id), int)
        self.assertGreater(rule_id, 0)

        data = self.client.get_rule(rule_id)
        # default should be True
        self.assertEqual(data['enable_rule'], True)
        # default should be True
        self.assertEqual(data['last'], True)

        condition[0]['val'] = 'JP'
        cache_key = [
            {'name': "uri", },
            {'name': "client-city"},
            {'name': "client-continent", 'args': 'first-x-forwarded-addr'},
            {'name': "query-string"}
        ]
        default_ttls= [{
            'ttl_unit': "min", 'status': 200, 'ttl': 2
        }, {
            'ttl_unit': "min", 'status': 301, 'ttl': 1
        }]
        cache_rule = {
            'cache_key': cache_key,
            'default_ttls': default_ttls,
            'browser_ttl': 2,
            'browser_ttl_unit': 'min',
            'disable_convert_head': True,
            'enforce_cache': True,
            'cluster_hash': True,
            'enable_global': ON,
        }
        ok = self.client.put_rule(rule_id=rule_id, condition=condition,
                                  cache=cache_rule)
        self.assertTrue(ok)

        data = self.client.get_rule(rule_id)
        self.assertEqual(data['cache']['cache_key'][0]['name'], 'uri')
        self.assertEqual(data['cache']['cache_key'][1]['name'], 'client-city')
        self.assertEqual(data['cache']['cache_key'][2]['args'], 'first-x-forwarded-addr')
        self.assertEqual(len(data['cache']['cache_key']), 4)
        self.assertEqual(data['cache']['cluster_hash'], True)
        self.assertEqual(data['cache']['disable_convert_head'], True)
        self.assertEqual(data['cache']['enforce_cache'], True)
        self.assertEqual(data['cache']['enable_global'], ON)
        self.assertEqual(data['cache']['browser_ttl'], 2)
        self.assertEqual(data['cache']['browser_ttl_unit'], 'min')
        self.assertEqual(type(data['cache']['default_ttls']), type(list()))
        self.assertEqual(data['conditions'][0]['values'][0]['val'], 'JP')

        # disable default cache
        # disable browser cache
        cache_rule = {
            'cache_key': cache_key,
            'default_ttls': OFF,
            'browser_ttl': OFF,
            'browser_ttl_unit': 'min',
            'disable_convert_head': False,
            'enforce_cache': False,
            'cluster_hash': False,
            'enable_global': OFF,
        }
        ok = self.client.put_rule(rule_id=rule_id, condition=condition,
                                  cache=cache_rule)
        self.assertTrue(ok)

        data = self.client.get_rule(rule_id)
        self.assertEqual(data['cache'].get('default_ttls',None), None)
        self.assertEqual(data['cache']['cluster_hash'], False)
        self.assertEqual(data['cache']['enforce_cache'], False)
        self.assertEqual(data['cache'].get('browser_ttl',None), None)
        self.assertEqual(data['cache'].get('browser_ttl_unit',None), None)
        self.assertEqual(data['cache']['disable_convert_head'], False)
        self.assertEqual(data['cache']['enable_global'], OFF)

        ok = self.client.del_rule(rule_id)
        self.assertTrue(ok)

        data = self.client.get_rule(rule_id)
        self.assertEqual(data, {})

    def test_content(self):
        condition = [{'var': 'uri', 'op': 'eq', 'val': '/favicon.ico'}]
        file_id = self.client.upload_favicon(name='test',
                                             favicon_content='content')
        self.assertIs(type(file_id), int)
        self.assertGreater(file_id, 0)

        rule_id = self.client.new_rule(condition=condition,
                                       content={'favicon': file_id})
        self.assertIs(type(rule_id), int)
        self.assertGreater(rule_id, 0)

        new_file_id = self.client.upload_favicon(
            name='test',
            favicon_content='new_content')
        self.assertIs(type(new_file_id), int)
        self.assertGreater(new_file_id, 0)

        ok = self.client.put_rule(rule_id=rule_id,
                                  content={'favicon': new_file_id})
        self.assertTrue(ok)

        data = self.client.get_rule(rule_id)

        self.assertEqual(data['content']['file'], new_file_id)
        self.assertEqual(data['conditions'][0]['values'][0]['val'],
                         '/favicon.ico')

        ok = self.client.del_rule(rule_id)
        self.assertTrue(ok)

        ok = self.client.del_favicon(file_id)
        ok = self.client.del_favicon(new_file_id)
        self.assertTrue(ok)

        data = self.client.get_rule(rule_id)
        self.assertEqual(data, {})

    def test_empty_gif(self):
        condition = [{'var': 'uri', 'op': 'eq', 'val': '/'}]
        rule_id = self.client.new_rule(condition=condition,
                                       content={'empty_gif': True})
        self.assertIs(type(rule_id), int)
        self.assertGreater(rule_id, 0)

        data = self.client.get_rule(rule_id)
        self.assertTrue(data['content']['empty_gif'])

        ok = self.client.del_rule(rule_id)
        self.assertTrue(ok)

        data = self.client.get_rule(rule_id)
        self.assertEqual(data, {})

    def test_get_all_rules(self):
        condition = [{'var': 'client-country', 'op': 'eq', 'val': 'CN'}]
        cache_key = ['uri', 'query-string', 'client-city']
        cache_rule = {'cache_key': cache_key}
        rule_id = self.client.new_rule(condition=condition, cache=cache_rule,
                                       top=1)
        self.assertIs(type(rule_id), int)
        self.assertGreater(rule_id, 0)

        condition = [
            {'var': 'host', 'val': 'con.' + self.apex},
            {'var': ['req-header', 'Referer'],
             'vals': [[r'foo\d+', 'rx'], 'foo.com']}
        ]
        conseq = {
            'enable-websocket': {},
            'redirect': {'url': '/cn/2017/', 'code': 302},
            'user-code': {'el': 'true => say(\"hello\");'}
        }
        rule_id = self.client.new_rule(condition=condition, conseq=conseq,
                                       last=True)
        self.assertIs(type(rule_id), int)
        self.assertGreater(rule_id, 0)

        data = self.client.get_all_rules()
        self.assertEqual(len(data), 2)

        self.assertEqual(data[0]['top'], 1)
        self.assertTrue(data[1]['last'])

        data = self.client.get_all_rules_by_app_domain(self.apex)
        self.assertEqual(len(data), 2)

    def test_el(self):
        code = "true => print('hello, {}');".format(self.apex)
        ok = self.client.new_el(phase='req-rewrite', code=code, pre=True)
        self.assertTrue(ok)

    def test_cert(self):
        domain = str(random.randint(1, 10000)) + '.foo.com'
        ok = self.client.put_app(app_id=self.app_id, domains=[domain],
                                 label='foo.com')
        self.assertTrue(ok)

        self.client.new_release()

        key_file = os.path.join(CURPATH, 'tests', 'key.pem')
        cert_file = os.path.join(CURPATH, 'tests', 'cert.pem')
        with open(key_file) as f:
            key = f.read()
        with open(cert_file) as f:
            cert = f.read()

        cert_id = self.client.set_cert_key(key=key, cert=cert)
        self.assertIs(type(cert_id), int)
        self.assertGreater(cert_id, 0)

        data = self.client.get_cert_key(cert_id)
        self.assertEqual(data['domains'][0], '*.foo.com')

        data = self.client.get_all_cert_keys()
        self.assertEqual(len(data), 1)
        self.assertEqual(data[0]['domains'], ['*.foo.com'])

        ok = self.client.del_cert_key(cert_id)
        self.assertTrue(ok)

        data = self.client.get_cert_key(cert_id)
        self.assertEqual(data, {})

    def test_req_rewrite_3(self):
        condition = [
            {'var': 'host', 'val': 'con.' + self.apex},
            {'var': 'server-port', 'op': 'is-empty'},
        ]
        conseq = {
            'enable-websocket': {},
            'redirect': {'url': '/cn/2017/', 'code': 302}
        }
        rule_id = self.client.new_rule(condition=condition, conseq=conseq)
        self.assertIs(type(rule_id), int)
        self.assertGreater(rule_id, 0)

        data = self.client.get_rule(rule_id)
        self.assertEqual(data['conditions'][0]['variable']['name'], 'host')
        self.assertEqual(data['conditions'][1]['variable']['name'],
                         'server-port')
        self.assertEqual(data['conditions'][1]['operator']['name'], 'is-empty')

        ok = self.client.del_rule(rule_id)
        self.assertTrue(ok)
        data = self.client.get_rule(rule_id)
        self.assertEqual(data, {})

    def test_rule_order(self):
        condition = [{'var': 'uri', 'op': 'eq', 'val': '/favicon.ico'}]
        file_id = self.client.upload_favicon(name='test',
                                             favicon_content='content')
        self.assertIs(type(file_id), int)
        self.assertGreater(file_id, 0)

        rule_id = self.client.new_rule(condition=condition,
                                       content={'favicon': file_id})
        self.assertIs(type(rule_id), int)
        self.assertGreater(rule_id, 0)

        new_file_id = self.client.upload_favicon(
            name='test',
            favicon_content='new_content')
        self.assertIs(type(new_file_id), int)
        self.assertGreater(new_file_id, 0)

        ok = self.client.put_rule(rule_id=rule_id,
                                  content={'favicon': new_file_id},
                                  order=1)
        self.assertTrue(ok)

        data = self.client.get_rule(rule_id)
        self.assertEqual(data['order'], 1)

        ok = self.client.del_rule(rule_id)
        self.assertTrue(ok)

        ok = self.client.del_favicon(file_id)
        ok = self.client.del_favicon(new_file_id)
        self.assertTrue(ok)

    def test_disable_rule(self):
        conseq = {
            'enable-websocket': {},
            'redirect': {'url': '/cn/2017/', 'code': 302}
        }
        rule_id = self.client.new_rule(conseq=conseq)
        self.assertIs(type(rule_id), int)
        self.assertGreater(rule_id, 0)

        data = self.client.get_rule(rule_id)
        # default should be True
        self.assertEqual(data['enable_rule'], True)
        # default should be False
        self.assertEqual(data['last'], False)

        rule_id = self.client.new_rule(conseq=conseq, enable=False)
        self.assertIs(type(rule_id), int)
        self.assertGreater(rule_id, 0)

        data = self.client.get_rule(rule_id)
        self.assertEqual(data['enable_rule'], False)

        ok = self.client.put_rule(rule_id, conseq=conseq, enable=True)
        self.assertEqual(ok, True)
        data = self.client.get_rule(rule_id)
        self.assertEqual(data['enable_rule'], True)

        ok = self.client.put_rule(rule_id, conseq=conseq, enable=False)
        self.assertEqual(ok, True)
        data = self.client.get_rule(rule_id)
        self.assertEqual(data['enable_rule'], False)


    def test_disable_sections(self):
        condition = [{'var': 'client-country', 'op': 'eq', 'val': 'CN'}]
        waf_rule = {'rule_sets': [7,8,9,10,11,12,13,14,15,16,17],
                    'action': '403 Forbidden', 'threshold': 'low'}
        conseq = {
            'enable-websocket': {},
            'redirect': {'url': '/cn/2017/', 'code': 302},
        }
        cache_key = ['uri', 'query-string', 'client-city']
        cache_rule = {'cache_key': cache_key}

        condition = [{'var': 'uri', 'op': 'eq', 'val': '/favicon.ico'}]
        file_id = self.client.upload_favicon(name='test2', favicon_content='content')
        self.assertIs(type(file_id), int)
        self.assertGreater(file_id, 0)

        up_id = self.client.new_upstream(
            name='origin-upstream',
            servers=[
                {'domain': 'test.com', 'port': 80},
                {'ip': '172.22.31.2', 'port': 80, 'weight': 2}
            ])
        self.assertIs(type(up_id), int)
        self.assertGreater(up_id, 0)

        backup_up_id = self.client.new_upstream(
            name='backup-upstream',
            servers=[
                {'ip': '172.22.31.3', 'port': 80},
                {'ip': '172.22.31.4', 'port': 80, 'weight': 3}
            ])
        self.assertIs(type(backup_up_id), int)
        self.assertGreater(backup_up_id, up_id)

        proxy_rule = {
            'upstreams': [{'upstream': up_id, 'weight': 2}],
            'backup_upstreams': [{'upstream': backup_up_id}],
            'retries': 2,
            "retry_condition": ["invalid_header", "http_500"],
            "balancer_algorithm": "hash",
            "balancer_vars": [
                {"name": "uri"},
                {
                    "name": "uri-arg",
                    "args": "foo"
                }
            ]
        }

        rule_id = self.client.new_rule(condition=condition,
                                       conseq=conseq,
                                       waf=waf_rule,
                                       cache=cache_rule,
                                       proxy=proxy_rule,
                                       content={'favicon': file_id})

        data = self.client.get_rule(rule_id)
        self.assertEqual(data['enable_rule'], True)
        self.assertEqual(len(data['conditions']), 1)
        self.assertEqual(len(data['actions']), 2)
        self.assertGreater(len(data['cache']), 1)
        self.assertGreater(len(data['content']), 1)
        self.assertGreater(len(data['proxy']), 1)
        self.assertEqual(len(data['proxy']['backup_upstream']), 1)
        self.assertEqual(len(data['proxy']['upstream']), 1)
        self.assertGreater(len(data['waf']), 1)

        proxy_rule['backup_upstreams'] = None

        ok = self.client.put_rule(rule_id, proxy=proxy_rule)
        self.assertEqual(ok, True)

        data = self.client.get_rule(rule_id)
        self.assertEqual(data['enable_rule'], True)
        self.assertEqual(len(data['conditions']), 1)
        self.assertEqual(len(data['actions']), 2)
        self.assertGreater(len(data['cache']), 1)
        self.assertGreater(len(data['content']), 1)
        self.assertGreater(len(data['proxy']), 1)
        self.assertEqual(data['proxy'].get('backup_upstream'), None)
        self.assertEqual(len(data['proxy']['upstream']), 1)
        self.assertGreater(len(data['waf']), 1)

        ok = self.client.put_rule(rule_id, condition=OFF,
                                       conseq=conseq,
                                       waf=OFF,
                                       cache=OFF,
                                       proxy=OFF,
                                       content=OFF)

        self.assertEqual(ok, True)

        data = self.client.get_rule(rule_id)
        self.assertEqual(data['enable_rule'], True)
        self.assertEqual(data.get('conditions', None), None)
        self.assertEqual(len(data['actions']), 2)
        self.assertEqual(data.get('cache', None), None)
        self.assertEqual(data.get('content', None), None)
        self.assertEqual(data.get('proxy', None), None)
        self.assertEqual(data.get('waf', None), None)

        ok = self.client.put_rule(rule_id, conseq=OFF,
                                       waf=waf_rule)

        self.assertEqual(ok, True)

        data = self.client.get_rule(rule_id)
        self.assertEqual(data['enable_rule'], True)
        self.assertEqual(data.get('conditions', None), None)
        self.assertEqual(data.get('actions', None), None)
        self.assertEqual(data.get('cache', None), None)
        self.assertEqual(data.get('content', None), None)
        self.assertEqual(data.get('proxy', None), None)
        self.assertGreater(len(data['waf']), 1)


    def test_rule_last(self):
        conseq = {
            'enable-websocket': {},
            'redirect': {'url': '/cn/2017/', 'code': 302}
        }
        rule_id = self.client.new_rule(conseq=conseq)
        self.assertIs(type(rule_id), int)
        self.assertGreater(rule_id, 0)

        conseq = {
            'redirect': {'url': '/cn/2018/', 'code': 301}
        }
        rule_id2 = self.client.new_rule(conseq=conseq)
        self.assertIs(type(rule_id), int)
        self.assertGreater(rule_id, 0)

        rules = self.client.get_all_rules()
        self.assertEqual(len(rules), 2)
        self.assertGreater(rules[0]['id'], rules[1]['id'])

        ok = self.client.put_rule(rule_id2, conseq=conseq, last=True)
        self.assertEqual(ok, True)
        data = self.client.get_rule(rule_id2)
        self.assertEqual(data['last'], True)

        # the "last" tag is unrelated to sequence,
        # it is just a switch in the page rules.
        self.assertGreater(rules[0]['id'], rules[1]['id'])

    def test_rule_order2(self):
        conseq = {
            'enable-websocket': {},
            'redirect': {'url': '/cn/2017/', 'code': 302}
        }
        rule_id = self.client.new_rule(conseq=conseq)
        self.assertIs(type(rule_id), int)
        self.assertGreater(rule_id, 0)

        conseq = {
            'redirect': {'url': '/cn/2018/', 'code': 301}
        }
        rule_id2 = self.client.new_rule(conseq=conseq)
        self.assertIs(type(rule_id), int)
        self.assertGreater(rule_id, 0)

        rules = self.client.get_all_rules()
        self.assertEqual(len(rules), 2)
        # rule_id == rules[1]['id']
        # rule_id2 == rules[0]['id']
        self.assertGreater(rules[0]['id'], rules[1]['id'])

        ok = self.client.put_rule(rule_id2, conseq=conseq, order=1)
        self.assertEqual(ok, True)
        data = self.client.get_rule(rule_id2)
        self.assertEqual(data['order'], 1)

        rules = self.client.get_all_rules()
        self.assertGreater(rules[1]['id'], rules[0]['id'])

        # reset to 0
        ok = self.client.put_rule(rule_id2, conseq=conseq, order=0)
        self.assertEqual(ok, True)
        data = self.client.get_rule(rule_id2)
        self.assertEqual(data['order'], 0)

        rules = self.client.get_all_rules()
        self.assertGreater(rules[0]['id'], rules[1]['id'])


    def test_rule_comment(self):
        conseq = {
            'enable-websocket': {},
            'redirect': {'url': '/cn/2017/', 'code': 302}
        }
        rule_id = self.client.new_rule(conseq=conseq)
        self.assertIs(type(rule_id), int)
        self.assertGreater(rule_id, 0)

        data = self.client.get_rule(rule_id)
        self.assertEqual(data.get('comment', None), None)

        rule_id = self.client.new_rule(conseq=conseq, comment="test")
        self.assertIs(type(rule_id), int)
        self.assertGreater(rule_id, 0)

        data = self.client.get_rule(rule_id)
        self.assertEqual(data.get('comment', None), "test")

        ok = self.client.put_rule(rule_id, conseq=conseq, comment="test2")
        self.assertEqual(ok, True)

        data = self.client.get_rule(rule_id)
        self.assertEqual(data.get('comment', None), "test2")

        ok = self.client.put_rule(rule_id, conseq=conseq, comment="")
        self.assertEqual(ok, True)

        data = self.client.get_rule(rule_id)
        self.assertEqual(data.get('comment', None), None)

    def test_waf_rule_set_name(self):
        condition = [{'var': 'uri', 'op': 'prefix', 'val': '/foo'}]
        waf_rule = {'rule_sets': ["protocol_enforcement", "protocol_attack"],
                    'action': '403 Forbidden', 'threshold': 'high'}
        rule_id = self.client.new_rule(condition=condition,
                                       waf=waf_rule, last=True)
        self.assertIs(type(rule_id), int)
        self.assertGreater(rule_id, 0)

        data = self.client.get_rule(rule_id)
        self.assertIn(7, data['waf']['rule_sets'])
        self.assertIn(8, data['waf']['rule_sets'])
        self.assertEqual(data['waf']['threshold_score'], 1000)



# -*- coding: utf-8 -*-
# import io
import os
import random
import sdk_test
from edge2client.constants import OFF

CURPATH = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))


class TestRule(sdk_test.TestSdk):
    def test_cache(self):
        condition = [{'var': 'client-country', 'op': 'eq', 'val': 'CN'}]
        cache_key = ['uri', 'query-string', 'client-city']
        cache_rule = {'cache_key': cache_key}
        rule_id = self.client.new_rule(condition=condition, cache=cache_rule)
        self.assertIs(type(rule_id), int)
        self.assertGreater(rule_id, 0)

        condition[0]['val'] = 'JP'
        cache_key = [
            {'name': "uri", },
            {'name': "client-city"},
            {'name': "client-continent", 'args': 'first-x-forwarded-addr'},
            {'name': "query-string"}
        ]
        default_ttls= [{
            'ttl_unit': "min", 'status': 200, 'ttl': 2
        }, {
            'ttl_unit': "min", 'status': 301, 'ttl': 1
        }]
        cache_rule = {
            'cache_key': cache_key,
            'default_ttls': default_ttls,
            'browser_ttl': 2,
            'browser_ttl_unit': 'min',
            'disable_convert_head': True,
            'enforce_cache': True,
            'cluster_hash': True,
            'enable_global': True,
        }
        ok = self.client.put_rule(rule_id=rule_id, condition=condition,
                                  cache=cache_rule)
        self.assertTrue(ok)

        data = self.client.get_rule(rule_id)
        self.assertEqual(data['cache']['cache_key'][0]['name'], 'uri')
        self.assertEqual(data['cache']['cache_key'][1]['name'], 'client-city')
        self.assertEqual(data['cache']['cache_key'][2]['args'], 'first-x-forwarded-addr')
        self.assertEqual(len(data['cache']['cache_key']), 4)
        self.assertEqual(data['cache']['cluster_hash'], True)
        self.assertEqual(data['cache']['disable_convert_head'], True)
        self.assertEqual(data['cache']['enforce_cache'], True)
        self.assertEqual(data['cache']['enable_global'], True)
        self.assertEqual(data['cache']['browser_ttl'], 2)
        self.assertEqual(data['cache']['browser_ttl_unit'], 'min')
        self.assertEqual(type(data['cache']['default_ttls']), type(list()))
        self.assertEqual(data['conditions'][0]['values'][0]['val'], 'JP')

        # disable default cache
        cache_rule = {
            'cache_key': cache_key,
            'default_ttls': OFF,
            'browser_ttl': OFF,
            'browser_ttl_unit': 'min',
            'disable_convert_head': False,
            'enforce_cache': False,
            'cluster_hash': False,
            'enable_global': False,
        }
        ok = self.client.put_rule(rule_id=rule_id, condition=condition,
                                  cache=cache_rule)
        self.assertTrue(ok)

        data = self.client.get_rule(rule_id)
        self.assertEqual(data['cache'].get('default_ttls',None), None)
        self.assertEqual(data['cache']['cluster_hash'], False)
        self.assertEqual(data['cache']['enforce_cache'], False)
        self.assertEqual(data['cache'].get('browser_ttl',None), None)
        self.assertEqual(data['cache'].get('browser_ttl_unit',None), None)
        self.assertEqual(data['cache']['disable_convert_head'], False)
        self.assertEqual(data['cache']['enable_global'], False)

        ok = self.client.del_rule(rule_id)
        self.assertTrue(ok)

        data = self.client.get_rule(rule_id)
        self.assertEqual(data, {})



import os

from .utils import error, warn, info, is_valid_host, \
    is_valid_ipv4_address, is_valid_ipv6_address, line
from .read_config import read_yaml_config, write_yaml_config
from .k8s_upstreams import check_upstream_checker

def process_global_upstreams(ctx):
    client = ctx['client']
    configs_path = ctx['configs_path']
    location = ctx.get('location', None)

    if not location:
        location = "global_upstreams"

    configs = read_yaml_config(configs_path, location)
    if configs is None:
        return

    info("Checking if global upstreams are valid")
    for filename, upstreams in configs.items():
        check_global_upstreams(upstreams, filename)

    old_upstreams = client.get_all_global_upstreams(detail=True)

    old_upstreams_dict = {upstream['name']: upstream for upstream in old_upstreams}

    info("Checking if global upstreams have changed")
    new_upstreams_dict = {}
    sorted_configs = sorted(configs.keys())

    for filename in sorted_configs:
        upstreams = configs[filename]
        for up_name, up in upstreams.items():
            new_upstreams_dict[up_name] = True

            checker = None
            if up.get('enable_checker', False):
                checker = up.get('checker', dict())

            if up_name in old_upstreams_dict:
                old_up = old_upstreams_dict[up_name]
                if is_change_global_upstream(up, old_up):
                    try:
                        info(f"Updating Global upstream \"{up_name}\"")
                        client.put_global_upstream(
                            up_id=old_up['id'],
                            name=up_name,
                            ssl=up.get('ssl', False),
                            health_checker=checker,
                            disable_ssl_verify=up.get('disable_ssl_verify', False),
                            servers=format_upstream_server(up['servers'])
                        )
                    except Exception as e:
                        error(f"Failed to update global upstream, file: {filename}, line: {line(up)}", e)
            else:
                try:
                    info(f"Adding global upstream \"{up_name}\"")
                    client.new_global_upstream(
                        name=up_name,
                        ssl=up.get('ssl', False),
                        disable_ssl_verify=up.get('disable_ssl_verify', False),
                        health_checker=checker,
                        servers=format_upstream_server(up['servers'])
                    )
                except Exception as e:
                    error(f"Failed to add global upstream, file: {filename}, line: {line(up)}", e)

    # since this is a global configuration,
    # we will not perform deletion operations in order to maintain compatibility with multiple local configurations.
    # for up_name, up in old_upstreams_dict.items():
    #     if up_name not in new_upstreams_dict:
    #         try:
    #             info(f"Removing Global upstream \"{up_name}\"")
    #             client.del_global_upstream(up['id'])
    #         except Exception as e:
    #             error(f"Failed to remove Global upstream, upstream id: {up['id']}", e)

def cleanup_global_upstreams(ctx):
    pass
    # since this is a global configuration,
    # we will not perform deletion operations in order to maintain compatibility with multiple local configurations.
    # client = ctx['client']

    # upstreams = client.get_all_global_upstreams()

    # for upstream in upstreams:
    #     try:
    #         info(f"Removing Global upstream \"{upstream['name']}\"")
    #         client.del_global_upstream(upstream['id'])
    #     except Exception as e:
    #         error(f"Failed to remove Global upstream, upstream id: {upstream['id']}", e)

def export_global_upstreams(ctx):
    client = ctx['client']
    configs_path = ctx['export_to_path']

    upstreams = client.get_all_global_upstreams(detail=True)

    if not upstreams:
        info(f"No Global upstreams found")
        return

    formatted_upstreams = {}
    for upstream in upstreams:
        formatted_upstream = {
            'ssl': upstream.get('ssl', False),
            'disable_ssl_verify': upstream.get('disable_ssl_verify', False),
            'enable_checker': upstream.get('enable_checker', False),
            'servers': []
        }

        if formatted_upstream['enable_checker']:
            formatted_upstream['checker'] = upstream.get('checker', {})

        for node in upstream.get('nodes', []):
            server = {}
            if 'ip' in node:
                server['host'] = node['ip']
            elif 'domain' in node:
                server['host'] = node['domain']
            else:
                continue

            server['port'] = node.get('port')
            formatted_upstream['servers'].append(server)

        formatted_upstreams[upstream['name']] = formatted_upstream

    export_path = os.path.join(configs_path, "global_upstreams")

    try:
        write_yaml_config(export_path, "global_upstreams.yaml", formatted_upstreams)
        info(f"Global upstreams exported successfully to global_upstreams/global_upstreams.yaml")
    except Exception as e:
        error(f"Failed to export global upstreams to global_upstreams/global_upstreams.yaml", e)

def check_global_upstreams(upstreams, filename):
    if not isinstance(upstreams, dict):
        error(f"Unsupported global upstream file format, file: {filename}")

    for up_name, up in upstreams.items():
        if not isinstance(up_name, str):
            error(f"Unsupported global upstream name format, file: {filename}, line: {line(up_name)}")

        if not isinstance(up, dict):
            error(f"Global upstream for name {up_name} must be a dict, file: {filename}, line: {line(up)}")

        ssl = up.get('ssl', False)
        if not isinstance(ssl, bool):
            error(f"ssl flag for global upstream {up_name} must be a boolean, file: {filename}, line: {line(ssl)}")

        disable_ssl_verify = up.get('disable_ssl_verify', False)
        if not isinstance(disable_ssl_verify, bool):
            error(f"disable_ssl_verify flag for global upstream {up_name} must be a boolean, file: {filename}, line: {line(disable_ssl_verify)}")

        enable_checker = up.get('enable_checker', False)
        if not isinstance(enable_checker, bool):
            error(f"enable_checker flag for global upstream {up_name} must be a boolean, file: {filename}, line: {line(enable_checker)}")

        if enable_checker:
            check_upstream_checker(up, up_name, filename)

        servers = up.get('servers', None)
        if not isinstance(servers, list) or not all(isinstance(item, dict) for item in servers):
            error(f"Global upstream servers for upstream {up_name} must be a list of dictionaries, file: {filename}, line: {line(servers)}")

        for s in servers:
            host = s.get("host", None)
            port = s.get("port", None)

            if not isinstance(host, str) or not is_valid_host(host):
                error(f"Invalid host in global upstream {up_name}: {host}, file: {filename}, line: {line(host)}")

            if not (isinstance(port, int) and 1 <= port <= 65535):
                error(f"Invalid port in global upstream {up_name}: {port}, file: {filename}, line: {line(port)}")

def is_change_global_upstream(new_up, old_up):
    if new_up.get('ssl', False) != old_up.get('ssl', False):
        return True

    if new_up.get('disable_ssl_verify', False) != old_up.get('disable_ssl_verify', False):
        return True

    if new_up.get('enable_checker', False) != old_up.get('enable_checker', False):
        return True

    if new_up.get('enable_checker', False) is True:
        new_checker = new_up.get('checker', dict())
        old_checker = old_up.get('checker', dict())
        for key in new_checker:
            if new_checker[key] != old_checker[key]:
                return True

    new_servers = new_up.get('servers', [])
    old_servers = old_up.get('nodes', [])
    if len(new_servers) != len(old_servers):
        return True

    for s1, s2 in zip(new_servers, old_servers):
        host = s1.get('host')
        if is_valid_ipv4_address(host) or is_valid_ipv6_address(host):
            if s1.get('host') != s2.get('ip', None):
                return True
        else:
            if s1.get('host') != s2.get('domain', None):
                return True

        if s1.get('port') != s2.get('port', None):
            return True

    return False

def format_upstream_server(servers):
    formatted_servers = []
    for s in servers:
        server = {}
        host = s.get('host')
        if is_valid_ipv4_address(host) or is_valid_ipv6_address(host):
            server['ip'] = host
        else:
            server['domain'] = host
        server['port'] = s.get('port')
        formatted_servers.append(server)
    return formatted_servers



import os

from .utils import error, warn, info, get_app_id_by_domain, line, is_partition_action
from .read_config import read_yaml_config, write_yaml_config, format_yaml_content
from .global_variables import replace_global_variable_in_el, restore_global_variable_names_in_el
from .global_page_templates import replace_global_page_template_in_el, restore_global_page_template_names_in_el
from .page_rules import export_conditions

def check_global_custom_actions(filename, actions):
    for action in actions:
        if 'name' not in action:
            error(f"Missing 'name' in global custom action, filename: {filename}, line: {line(action)}")

        if 'conditions' in action and not isinstance(action['conditions'], list):
            error(f"Invalid 'conditions' in global custom action {action['name']}, filename: {filename}, line: {line(action)}")

        if 'actions' not in action or not (isinstance(action['actions'], dict) or isinstance(action['actions'], list)):
            error(f"Missing or invalid 'actions' in global custom action {action['name']}, filename: {filename}, line: {line(action)}")

        if 'conditions' in action:
            for condition in action['conditions']:
                if 'var' not in condition:
                    error(f"Missing 'var' in condition, action: {action['name']}, filename: {filename}, line: {line(condition)}")
                if 'op' not in condition:
                    error(f"Missing 'op' in condition, action: {action['name']}, filename: {filename}, line: {line(condition)}")
                if 'vals' not in condition and 'val' not in condition:
                    error(f"Missing 'vals' or 'val' in condition, action: {action['name']}, filename: {filename}, line: {line(condition)}")

    return True

def get_all_global_actions(client, detail=False):
    return client.get_all_global_actions(detail=detail)

def is_changed(ctx, new_rule, old_rule):
    old_rule_actions = list()
    rule_actions = old_rule['actions']
    for rule_action in rule_actions:
        new_action = dict()
        if 'type' in rule_action and rule_action['type'] in rule_action:
            new_action[rule_action['type']] = rule_action[rule_action['type']]
        else:
            new_action[rule_action['type']] = dict()

        old_rule_actions.append(new_action)

    old_conditions = list()
    if "conditions" in old_rule:
        old_conditions = export_conditions(ctx, old_rule['conditions'])

    if new_rule['name'] != old_rule['name']:
        return True
    if new_rule.get("conditions", list()) != old_conditions:
        return True
    if new_rule['actions'] != old_rule_actions:
        return True

    return False

def process_global_custom_actions(ctx):
    client = ctx['client']
    partition_id = ctx['partition_id']
    configs_path = ctx['configs_path']
    location = ctx.get('location', None)

    if not location:
        location = "global_custom_actions"

    configs = read_yaml_config(configs_path, location)
    if not configs:
        return

    info("Checking if global custom actions are valid")

    sorted_filenames = sorted(configs.keys())

    for filename in sorted_filenames:
        config = configs[filename]
        check_global_custom_actions(filename, config)

    # Get existing global custom actions
    existing_actions = get_all_global_actions(client, True)
    existing_actions_dict = {action['name']: action for action in existing_actions}

    # Process actions
    for filename in sorted_filenames:
        actions = configs[filename]
        for action in actions:
            name = action['name']
            conditions = action.get('conditions', None)
            actions_config = action['actions']

            replace_global_variable_in_el(client, actions_config, filename, partition_id)
            replace_global_page_template_in_el(client, actions_config, filename)

            if name in existing_actions_dict:
                if is_changed(ctx, action, existing_actions_dict[name]):
                    # Update existing action
                    try:
                        info(f"Updating global custom action \"{name}\", partition id: {partition_id}")
                        client.put_global_action(
                            name=name,
                            action_id=existing_actions_dict[name]['id'],
                            condition=conditions,
                            conseq=actions_config
                        )
                    except Exception as e:
                        error(f"Failed to update global custom action, file: {filename}, line: {line(action)}", e)
            else:
                # Add new action
                try:
                    info(f"Adding global custom action \"{name}\" to partition, partition id: {partition_id}")
                    client.new_global_action(
                        name=name,
                        condition=conditions,
                        conseq=actions_config
                    )
                except Exception as e:
                    error(f"Failed to add global custom action to partition, file: {filename}, line: {line(action)}", e)

            # Remove processed action from existing_actions_dict
            existing_actions_dict.pop(name, None)

    # since this is a global configuration,
    # we will not perform deletion operations in order to maintain compatibility with multiple local configurations.
    # Remove actions that are no longer in the config
    # for name, action in existing_actions_dict.items():
    #     try:
    #         info(f"Removing global custom action \"{name}\" from partition, partition id: {partition_id}")
    #         client.del_global_action(action['id'])
    #     except Exception as e:
    #         error(f"Failed to remove global custom action from partition, partition id: {partition_id}, action id: {action['id']}", e)

def cleanup_global_custom_actions(ctx):
    pass
    # since this is a global configuration,
    # we will not perform deletion operations in order to maintain compatibility with multiple local configurations.
    # client = ctx['client']
    # partition_id = ctx['partition_id']

    # actions = get_all_global_actions(client)

    # for action in actions:
    #     try:
    #         info(f"Removing global custom action \"{action['name']}\" from partition, partition id: {partition_id}")
    #         client.del_global_action(action['id'])
    #     except Exception as e:
    #         error(f"Failed to remove global custom action from partition, partition id: {partition_id}, action id: {action['id']}", e)

def export_global_custom_actions(ctx):
    client = ctx['client']
    partition_id = ctx['partition_id']
    configs_path = ctx['export_to_path']

    actions = get_all_global_actions(client, True)

    if not actions:
        info(f"No global custom actions found for partition_id: {partition_id}")
        return

    formatted_actions = []
    for action in actions:
        new_actions = list()
        rule_actions = action['actions']
        restore_global_variable_names_in_el(client, rule_actions, partition_id)
        restore_global_page_template_names_in_el(client, rule_actions)
        for rule_action in rule_actions:
            new_action = dict()
            if 'type' in rule_action and rule_action['type'] in rule_action:
                new_action[rule_action['type']] = rule_action[rule_action['type']]

                if rule_action['type'] == 'user-code':
                    ac = rule_action[rule_action['type']]
                    if 'el' in ac:
                        ac['el'] = format_yaml_content(ac['el'])
            else:
                new_action[rule_action['type']] = dict()

            new_actions.append(new_action)

        formatted_action = {
            'name': action['name'],
            'actions': new_actions
        }

        if 'conditions' in action:
            formatted_action['conditions'] = export_conditions(ctx, action['conditions'])

        formatted_actions.append(formatted_action)

    export_path = os.path.join(configs_path, 'global_custom_actions')

    try:
        write_yaml_config(export_path, "global_custom_actions.yaml", formatted_actions)
        info(f"Global custom actions exported successfully to global_custom_actions/global_custom_actions.yaml")
    except Exception as e:
        error(f"Failed to export global custom actions to global_custom_actions/global_custom_actions.yaml", e)



# -*- coding: utf-8 -*-
# import io
import sdk_test


class TestNodes(sdk_test.TestSdk):
    def test_get_all_nodes(self):
        data=self.client.get_all_nodes()
        self.assertEqual(type(data), type(list()))

        if len(data) == 0:
            return

        node = data[0]
        self.assertEqual(type(node), type({}))
        self.assertEqual(type(node['id']), type(1))
        self.assertEqual(type(node['gateway_id']), type(1))
        node_id = node['id']
        gateway_id = node['gateway_id']

        # test get_node
        node = self.client.get_node(node_id)
        self.assertEqual(type(node), type({}))

        node = self.client.get_node(node_id, gateway_id=gateway_id)
        self.assertEqual(type(node), type({}))

        # test put_node
        ok = self.client.put_node(node_id, \
            gateway_id=gateway_id, name='test', \
            is_gray=True, status=2, external_ip='127.0.0.1', \
            external_ipv6='::1', internal_ip='127.0.0.1')

        self.assertEqual(ok, True)

        node = self.client.get_node(node_id, gateway_id=gateway_id)
        self.assertEqual(type(node), type({}))
        self.assertEqual(node['name'], 'test')
        self.assertEqual(node['is_gray'], True)
        self.assertEqual(node['status'], 2)
        self.assertEqual(node['external_ip'], '127.0.0.1')
        self.assertEqual(node['external_ipv6'], '::1')
        self.assertEqual(node['internal_ip'], '127.0.0.1')

        # test put_node 2
        ok = self.client.put_node(node_id, \
            gateway_id=gateway_id, name='test2', \
            is_gray=False, status=1, external_ip='127.0.0.2', \
            external_ipv6='::2', internal_ip='127.0.0.2')

        self.assertEqual(ok, True)

        node = self.client.get_node(node_id, gateway_id=gateway_id)
        self.assertEqual(type(node), type({}))
        self.assertEqual(node['name'], 'test2')
        self.assertEqual(node['is_gray'], False)
        self.assertEqual(node['status'], 1)
        self.assertEqual(node['external_ip'], '127.0.0.2')
        self.assertEqual(node['external_ipv6'], '::2')
        self.assertEqual(node['internal_ip'], '127.0.0.2')



from urllib.parse import urlparse
import requests

#
# NOTICE: please edit these following configurations.
#

EUREKA_ADDR = 'http://127.0.0.1:18080/eureka/apps/'

# key: Edge application domain
# value: eureka app array.
EUREKA_APPS = {
    'test.com': ['eureka-app-name-1', 'eureka-app-name-2'],
    'test-2.com': ['eureka-app-name-3']
}

#
# end of configurations
#

try:
    import py_client
except ImportError:
    print("Error: cannot import py_client.py")
    exit(1)


def get_upstreams_from_eureka(app_name, domain):
    upstreams = {}
    ssl = False
    interval = 30
    http_req_uri = '/health'

    r = requests.get(EUREKA_ADDR + app_name,
                     headers={'Content-Type': 'application/json',
                              'Accept': 'application/json'},
                     timeout=10)
    # print(r.text)
    response = r.json()
    application = response['application']
    upstreams['name'] = application['name']
    instances = application['instance']
    upstreams['servers'] = []
    upstreams['health_checker'] = {
        'http_req_uri': http_req_uri,
        'http_req_host': domain,
        'interval': 30,
        'interval_unit': 'sec',
        'timeout': 3,
        'fall': 3,
        'rise': 2,
        'valid_statuses': [200, 302],
        'report_interval': 3,
        'report_interval_unit': 'min'
    }

    for instance in instances:
        server = {}
        server['ip'] = instance['ipAddr']
        port = instance['port']
        secure_port = instance['securePort']
        if port['@enabled'] == 'true':
            server['port'] = port['$']
        elif secure_port['@enabled'] == 'true':
            server['port'] = port['$']
            ssl = True
        interval = instance['leaseInfo']['renewalIntervalInSecs']
        http_req_uri = urlparse(instance['healthCheckUrl']).path
        upstreams['servers'].append(server)

    upstreams['ssl'] = ssl
    upstreams['health_checker']['interval'] = interval
    upstreams['health_checker']['http_req_uri'] = http_req_uri

    return upstreams


def main():
    print('begin to sync from eureka')

    client = py_client.get_client()

    for domain, apps in EUREKA_APPS.items():
        has_changes = False
        eureka_upstream_names = []

        print('sync the upstream of ' + domain)
        data = client.search_app(app_domain = domain)
        if not data:
            print("ERROR: can not found app: " + domain)
            continue

        app_id = data[0]['id']
        client.use_app(app_id)
        exist_upstreams = client.get_all_upstreams()
        # print(exist_upstreams)

        for eureka_app in apps:
            eureka_upstream = get_upstreams_from_eureka(eureka_app, domain)
            if not eureka_upstream:
                print("ERROR: can not found upstreams in eureka, app is: "
                      + eureka_app)
                continue
            # print(eureka_upstream)

            name = eureka_upstream['name']
            eureka_upstream_names.append(name)

            if name not in exist_upstreams:
                client.new_upstream(
                    name=name, servers=eureka_upstream['servers'],
                    health_checker=eureka_upstream['health_checker'],
                    ssl=eureka_upstream['ssl'])
                has_changes = True
                print('add new upstream, name: ' + name)

            else:
                data = client.get_upstream(exist_upstreams[name])
                # print(data)
                need_update = False

                if data['checker']['http_req_uri'] != \
                    eureka_upstream['health_checker']['http_req_uri'] \
                    or data['checker']['interval'] != \
                    eureka_upstream['health_checker']['interval'] \
                        or data['ssl'] != eureka_upstream['ssl']:
                    need_update = True

                nodes = data['nodes']
                nodes = sorted(nodes, key=lambda node: node['ip'])
                eureka_nodes = sorted(eureka_upstream['servers'],
                                      key=lambda node: node['ip'])
                if len(nodes) != len(eureka_nodes):
                    need_update = True

                nodes_len = len(nodes)
                for i in range(nodes_len):
                    if nodes[i]['ip'] != eureka_nodes[i]['ip'] \
                            or nodes[i]['port'] != eureka_nodes[i]['port']:
                        need_update = True
                        break

                if need_update:
                    client.put_upstream(
                        exist_upstreams[name], name=name,
                        servers=eureka_upstream['servers'],
                        health_checker=eureka_upstream['health_checker'],
                        ssl=eureka_upstream['ssl'])
                    has_changes = True
                    print('updated exist upstream, name: ' + name)

        # delete the upstream that not in the eureka.
        # for up_name, up_id in exist_upstreams.items():
        #     if up_name not in eureka_upstream_names:
        #         client.del_upstream(up_id)
        #         has_changes = True
        #         print('deleted upstream, name is ' + up_name)

        if has_changes:
            client.new_release()

    print('done')


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# import io
import sdk_test

class TestNodeMonitor(sdk_test.TestSdk):
    def test_node_monitor(self):
        # [{'system_net_packets_sent': 523.14583333333, 'system_CPU_percent': 44.854166666667, 'system_swap_total': 2147479552, 'system_CPU_loadavg_15m': 1.8879166666667, 'system_net_err_in': 0, 'system_CPU_loadavg_5m': 1.9847916666667, 'system_net_packets_recv': 498.85416666667, 'system_net_err_out': 0, 'system_swap_used': 6553600, 'system_CPU_loadavg_1m': 1.58, 'system_net_drop_out': 0, 'system_memory_used': 6050292053.3333, 'system_net_bytes_sent': 241699.75, 'system_memory_total': 12360978432, 'system_CPU_core': 8, 'system_net_drop_in': 0, 'system_net_bytes_recv': 312615.3125, 'node_utime': 1627462187, 'node_id': 1, 'max_id': 2822}]
        gateways = self.client.get_all_gateway()
        # print("gateways = ", gateways)

        for gateway in gateways:
            nodes = gateway.get('nodes', [])
            # print("nodes = ", nodes)
            for node in nodes:
                data = self.client.node_monitor(node['id'])
                self.assertIs(type(data), list)



#!/usr/bin/python
# -*- coding: UTF-8 -*-
"""
copy application rules.
"""

import argparse
from utils_common import proc_cond, proc_conseq, \
        proc_waf, proc_proxy, proc_cache, proc_content

try:
    import json
except ImportError:
    print('>>> WARN: cannot import json')

try:
    import py_client
except ImportError:
    print("Error: cannot import py_client.py")
    exit(1)


def clone_upstream(proxy, client, src_id, dst_id):
    dst_proxy = {
        'connect_timeout': proxy.get('connect_timeout'),
        'read_timeout': proxy.get('read_timeout'),
        'send_timeout': proxy.get('send_timeout'),
        'retries': proxy.get('retries'),
        'retry_condition': proxy.get('retry_condition'),
        'multi_tier': proxy.get('layer_policty'),
        'balancer_algorithm': proxy.get('balancer', {}).get('algorithm'),
        'balancer_vars': proxy.get('balancer', {}).get('variables'),
    }

    for field in ['upstream', 'backup_upstream']:
        dst_upstreams = []
        for upstream in proxy.get(field, []):
            client.use_app(src_id)
            upobj = client.get_upstream(upstream.get('cluster'))

            client.use_app(dst_id)
            servers = []
            for node in upobj.get('nodes'):
                servers.append({
                    'domain': node.get('domain'),
                    'ip': node.get('ip'),
                    'port': node.get('port'),
                    'weight': node.get('weight'),
                    'status': node.get('status')
                })
            health_checker = {}
            checker = upobj.get('checker')
            if checker and upobj.get('enable_checker'):
                for k in ['use_tcp', 'fall', 'http_req_host',
                          'http_req_uri', 'interval', 'interval_unit',
                          'report_interval', 'report_interval_unit',
                          'rise', 'timeout', 'valid_statuses',
                          'user_agent', 'http_ver']:
                    if checker.get(k) is not None:
                        health_checker[k] = checker.get(k)

            # XX: reuse upstream here
            upstreams = client.get_all_upstreams()

            if upstreams.get(upobj.get('name')):
                cluster_id = upstreams.get(upobj.get('name'))

            else:
                cluster_id = client.new_upstream(
                    name=upobj.get('name'),
                    servers=servers,
                    health_checker=health_checker,
                    ssl=upobj.get('ssl'),
                    group=upobj.get('group')
                )

            dst_upstreams.append({
                'cluster': cluster_id,
                'global_cluster': upstream.get('global_cluster', None),
                'weight': upstream.get('weight')
            })

        # update proxy upstream id
        if dst_upstreams:
            dst_proxy[field] = dst_upstreams

    return dst_proxy


def main(args):
    src_id, dst_id = args.from_id, args.to_id

    # src
    client = py_client.get_client()
    client.use_app(src_id)

    # app rules
    src_rules = client.get_all_rules()

    # check dst rules
    client.use_app(dst_id)
    dst_rules = client.get_all_rules()
    if len(dst_rules) > 0:
        print('>>> WARN: dst application already has rules. ')

        if not args.force:
            print('>>> ERR: please use --force option to overwrite the rules. '
                  'exiting...')
            exit(1)

        print('>>> WARN: overwriting the rules...')
        # clear the dst rules.
        for rule in dst_rules:
            client.del_rule(rule.get('id'))
        # clear the upstreams.
        all_upstreams = client.get_all_upstreams()
        for up_name in all_upstreams:
            client.del_upstream(all_upstreams[up_name])

    for rule in src_rules:
        if rule.get('actions') is None \
                and rule.get('waf') is None \
                and rule.get('cache') is None \
                and rule.get('content') is None \
                and rule.get('proxy') is None:
            continue

        proxy = rule.get('proxy')
        dst_proxy = None
        if proxy:
            dst_proxy = clone_upstream(proxy, client, src_id, dst_id)
            client.use_app(dst_id)

        client.new_rule(
                condition=proc_cond(rule.get('conditions', None)),
                conseq=proc_conseq(rule.get('actions')),
                waf=proc_waf(rule.get('waf')),
                cache=proc_cache(rule.get('cache')),
                proxy=proc_proxy(dst_proxy),
                content=proc_content(rule.get('content')),
                top=rule.get('top'),
                last=rule.get('last'),
                order=rule.get('order')
            )

    # check missing upstreams here
    client.use_app(src_id)
    src_upstreams = client.get_all_upstreams()

    client.use_app(dst_id)
    dst_upstreams = client.get_all_upstreams()

    for sup_name in src_upstreams:
        if dst_upstreams.get(sup_name):
            pass
        else:
            client.use_app(src_id)
            upobj = client.get_upstream(src_upstreams.get(sup_name))

            servers = []
            for node in upobj.get('nodes'):
                servers.append({
                    'domain': node.get('domain'),
                    'ip': node.get('ip'),
                    'port': node.get('port'),
                    'weight': node.get('weight'),
                    'status': node.get('status')
                })

            health_checker = {}
            checker = upobj.get('checker')
            if checker and upobj.get('enable_checker'):
                for k in ['use_tcp', 'fall', 'http_req_host',
                          'http_req_uri', 'interval', 'interval_unit',
                          'report_interval', 'report_interval_unit',
                          'rise', 'timeout', 'valid_statuses',
                          'user_agent', 'http_ver']:
                    if checker.get(k) is not None:
                        health_checker[k] = checker.get(k)

            client.use_app(dst_id)
            client.new_upstream(
                name=upobj.get('name'),
                servers=servers,
                health_checker=health_checker,
                ssl=upobj.get('ssl'),
                group=upobj.get('group')
            )

    if args.print_rules:
        print(json.dumps(client.get_all_rules()))

    # whitelist
    client.use_app(src_id)
    waf_whitelists = client.get_all_waf_whitelists()

    client.use_app(dst_id)
    for waf_whitelist in waf_whitelists:
        client.new_waf_whitelist(
            condition=proc_cond(waf_whitelist.get('conditions')),
            rule_sets=waf_whitelist.get('rule_sets'))

    if args.print_waf_whitelist:
        print(json.dumps(client.get_all_waf_whitelists()))

    # cert
    client.use_app(src_id)
    if client.get_all_cert_keys():
        print('>>> WARN: there are ssl certificate related configuration, '
              'please confirm.')


'''
    client.use_app(dst_id)

    if args.print_cert:
        print(json.dumps(client.get_all_cert_keys()))
'''


if __name__ == '__main__':
    parser = argparse.ArgumentParser(
            description='application copy')

    parser.add_argument('--dry-run',
                        dest='dry_run',
                        action='store_true',
                        help='dry run.')
    parser.add_argument('--force',
                        dest='force',
                        action='store_true',
                        help='overwrite the rules of dst application.')
    parser.add_argument('from_id', metavar='from-ID', type=int,
                        help='copy application from-ID')
    parser.add_argument('to_id', metavar='to-ID', type=int,
                        help='copy application to-ID')

    parser.add_argument('--print-rules', help=argparse.SUPPRESS,
                        dest='print_rules',
                        action='store_true')
    parser.add_argument('--print-waf-whitelist', help=argparse.SUPPRESS,
                        dest='print_waf_whitelist',
                        action='store_true')

    args = parser.parse_args()

    main(args)

'''
    parser.add_argument('--cert-key', type=str, dest='cert_key',
                        help='specify the certificate key file.')
    parser.add_argument('--print-cert',
                        dest='print_cert',
                        action='store_true')
'''



# -*- coding: utf-8 -*-
import os
import string
import random
import time
import subprocess
import sdk_test

CURPATH = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))

class TestWafLogs(sdk_test.TestSdk):

    def test_get_waf_logs(self):
        init_waf_log_script = os.path.join(CURPATH, 'tests', 'data', 'init-waf-logs.sh')

        app_id = self.app_id

        command = "/usr/bin/bash {} {}".format(init_waf_log_script, app_id)
        result = subprocess.run(command, shell=True, capture_output=True, text=True)
        print(result.stdout)
        if result.returncode != 0:
            print(result.stderr)
        self.assertEqual(result.returncode, 0)

        data, count = self.client.get_waf_logs(app_id)
        self.assertEqual(count, 3)
        self.assertEqual(len(data), count)

        data, count = self.client.get_waf_logs(app_id, show_all=True)
        self.assertEqual(count, 5)
        self.assertEqual(len(data), count)

        data, count = self.client.get_waf_logs(app_id, page=1, pagesize=2, show_all=True)
        self.assertEqual(count, 5)
        self.assertEqual(len(data), 2)

        data, count = self.client.get_waf_logs(app_id, page=2, pagesize=2, show_all=True)
        self.assertEqual(count, 5)
        self.assertEqual(len(data), 2)

        data, count = self.client.get_waf_logs(app_id, page=2, pagesize=2)
        self.assertEqual(count, 3)
        self.assertEqual(len(data), 1)

        current_time = time.time()

        start_time = current_time - 3600
        end_time = current_time + 3600
        data, count = self.client.get_waf_logs(app_id, start_time=start_time, end_time=end_time)
        self.assertEqual(count, 3)
        self.assertEqual(len(data), 3)

        start_time = current_time - 3600
        end_time = current_time - 1800
        data, count = self.client.get_waf_logs(app_id, start_time=start_time, end_time=end_time)
        self.assertEqual(count, 0)
        self.assertEqual(data, None)

        start_time = current_time + 1800
        end_time = current_time + 3600
        data, count = self.client.get_waf_logs(app_id, start_time=start_time, end_time=end_time)
        self.assertEqual(count, 0)
        self.assertEqual(data, None)

        host="test.com"
        data, count = self.client.get_waf_logs(app_id, host=host)
        self.assertEqual(count, 3)
        self.assertEqual(len(data), 3)

        host="test2.com"
        data, count = self.client.get_waf_logs(app_id, host=host)
        self.assertEqual(count, 0)
        self.assertEqual(data, None)

        header="Host: test.com"
        data, count = self.client.get_waf_logs(app_id, header=header)
        self.assertEqual(count, 3)
        self.assertEqual(len(data), 3)

        header="Host: test2.com"
        data, count = self.client.get_waf_logs(app_id, header=header)
        self.assertEqual(count, 0)
        self.assertEqual(data, None)

        rule_set_id=15
        data, count = self.client.get_waf_logs(app_id, rule_set_id=rule_set_id)
        self.assertEqual(count, 3)
        self.assertEqual(len(data), 3)

        rule_set_id=1
        data, count = self.client.get_waf_logs(app_id, rule_set_id=rule_set_id)
        self.assertEqual(count, 0)
        self.assertEqual(data, None)

        data, count = self.client.get_waf_logs(app_id, resp_status=403)
        self.assertEqual(count, 3)
        self.assertEqual(len(data), 3)

        data, count = self.client.get_waf_logs(app_id, resp_status=200)
        self.assertEqual(count, 0)
        self.assertEqual(data, None)

        action = "block"
        data, count = self.client.get_waf_logs(app_id, action=action)
        self.assertEqual(count, 3)
        self.assertEqual(len(data), 3)

        action = "edge-captcha"
        data, count = self.client.get_waf_logs(app_id, action=action)
        self.assertEqual(count, 0)
        self.assertEqual(data, None)

        remote_addr = "127.0.0.1"
        data, count = self.client.get_waf_logs(app_id, remote_addr=remote_addr)
        self.assertEqual(count, 3)
        self.assertEqual(len(data), 3)

        remote_addr = "127.0.0.2"
        data, count = self.client.get_waf_logs(app_id, remote_addr=remote_addr)
        self.assertEqual(count, 0)
        self.assertEqual(data, None)

        try:
            request_id = "00000580000455aad5e00000"
            data, count = self.client.get_waf_logs(app_id, request_id=request_id)
            self.assertEqual(count, 1)
            self.assertEqual(len(data), 1)
        except Exception as e:
            self.assertEqual(e.args[0], 'request id 00000580000455aad5e00000 does not belong to app {}, but belongs to 1'.format(app_id))



#!/usr/bin/python
# -*- coding: UTF-8 -*-
""" python style config file
This file is used to export common config
"""

from os import environ, getcwd, path
import sys

HOST = 'https://127.0.0.1:443'
USER = 'admin'
PASSWORD = 'edge@password'
SSL_VERIFY = False
PYTHON_SDK_DIR = path.join(getcwd(), '/usr/local/oredge-python-sdk/lib')
APPS_DIR = path.join(getcwd(), './apps')


def get_client():
    python_sdk_dir = environ.get('EDGE_PYTHON_DIR') \
        or PYTHON_SDK_DIR
    sys.path.append(python_sdk_dir)
    from edge2client import Edge2Client

    host = environ.get('EDGE_HOST') or HOST
    user = environ.get('EDGE_USER') or USER
    password = environ.get('EDGE_PASSWORD') or PASSWORD
    ssl_verify = environ.get('EDGE_SSL_VERIFY') or SSL_VERIFY

    print('--------- config file ----------\n'
          'edge-admin host: {}\n'
          'edge-admin user: {}\n'
          'edge-admin ssl_verify: {}\n'
          'edge-admin apps_dir: {}\n'
          'edge-admin python_sdk_dir: {}\n'
          '========= runtime =============='
          .format(host, user, ssl_verify, APPS_DIR, python_sdk_dir))

    client = Edge2Client(host, user, password)
    client.set_ssl_verify(ssl_verify)
    client.login()

    return client


def get_apps_dir():
    return APPS_DIR


if __name__ == '__main__':
    print('This statement will be executed only '
          'if this script is called directly')



#!/usr/bin/python
# -*- coding: UTF-8 -*-
"""
update app's upstream weight
"""

import argparse

try:
    import py_client
except ImportError:
    print("Error: cannot import py_client.py")
    exit(1)


def main(args):
    if not args.upstream_ip and not args.upstream_domain:
        print('--upstream_ip or --upstream_domain must be specified')
        exit(1)

    # init
    client = py_client.get_client()

    # search upstreams
    upstreams = []
    search_item = ''
    search_val = ''

    if args.upstream_ip:
        upstreams = client.search_app(upstream_ip=args.upstream_ip)
        search_item = 'ip'
        search_val = args.upstream_ip
    elif args.upstream_domain:
        upstreams = client.search_app(upstream_domain=args.upstream_domain)
        search_item = 'domain'
        search_val = args.upstream_domain

    nups = len(upstreams)

    upstream = {}
    upstream_id = args.upstream_id

    if nups == 0:
        print('upstream not found by {}: {}'.format(search_item, search_val))
        exit(1)
    elif nups == 1:
        upstream = upstreams[0]
    else:
        if not upstream_id:
            print('results are not uniq by {}: {}\n'
                  'please use --upstream_id to determine the upstream'
                  .format(search_item, search_val))

            for up in upstreams:
                app_name = ''
                app = up.get('app')
                if app:
                    app_name = app.get('name')

                print('id: {}, name: {}, app_name: {}'
                      .format(up.get('id'), up.get('name'), app_name))

            exit(1)

        for up in upstreams:
            if up.get('id') == upstream_id:
                upstream = up

        if not upstream or upstream.get('id') != upstream_id:
            print('upstream not found by upstream_id: {}'.format(upstream_id))
            exit(1)

    app = upstream.get('app')
    if not app:
        print('cannot update upstream without app')
        exit(1)

    app_id = app.get('id')
    client.use_app(app_id)

    servers = []

    upstream_obj = client.get_upstream(upstream.get('id'))
    old_servers = upstream_obj.get('nodes')
    for s in old_servers:
        server = {}
        for k in s:
            if k.startswith('_'):
                continue
            server[k] = s[k]

        # update weight here.
        val = server.get(search_item)
        if val and val == search_val:
            server['weight'] = args.weight

        servers.append(server)

    client.put_upstream(up_id=upstream.get('id'),
                        name=upstream.get('name'),
                        servers=servers)

    if not client.request_ok():
        print('update upstream weight failed: {}')
        exit(1)

    print('upstream id: {}, {}: "{}" update weight to "{}"'
          .format(upstream.get('id'), search_item, search_val, args.weight))


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='update upstream weight')

    parser.add_argument('--upstream-ip',
                        dest='upstream_ip',
                        help='update the weight according to the upstream ip')
    parser.add_argument('--upstream-domain',
                        dest='upstream_domain',
                        help='update the weight according to the'
                        ' upstream domain')
    parser.add_argument('--weight',
                        dest='weight',
                        type=int,
                        required=True,
                        help='upstream weight')
    parser.add_argument('--upstream-id',
                        dest='upstream_id',
                        type=int,
                        help='select a upstream according to the upstream_id')

    args = parser.parse_args()

    main(args)
